class Hash

  # Converts all keys in the Hash to be String values, returning a new Hash.
  # With a from_class parameter, limits conversion to only a certain class of keys.
  # It defaults to nil which convert any key class.
  #
  #   foo = { :name=>'Gavin', :wife=>:Lisa }
  #   foo.keys_to_iv    #=>  { "@name"=>"Gavin", "@wife"=>:Lisa }
  #   foo.inspect       #=>  { :name =>"Gavin", :wife=>:Lisa }
  #
  def keys_to_iv( of_class=nil )
    self.dup.keys_to_instance_vars!( of_class )
  end

  # Aliases for better Extensions and Rails compatibility
  alias_method( :variablize_keys, :keys_to_iv )

  # Synonym for Hash#keys_to_string, but modifies the receiver in place (and returns it).
  # With a from_class parameter, limits conversion to only a certain class of keys.
  # It defaults to nil which convert any key class.
  #
  #   foo = { :name=>'Gavin', :wife=>:Lisa }
  #   foo.keys_to_iv!   #=>  { "@name"=>"Gavin", "@wife"=>:Lisa }
  #   foo.inspect       #=>  { "@name"=>"Gavin", "@wife"=>:Lisa }
  #
  def keys_to_iv!( of_class=nil )
    raise ArgumentError, "Parameter must be a class" unless of_class.kind_of?(Class) if of_class
    if of_class
      self.each_key do |k|
        if k.respond_to?(:to_s) and k.class == of_class
          k = k.to_s
          nk = k[0,1] != '@' ? k : "@#{k}"
          self[nk]=self.delete(k)
        end
      end
    else
      self.each_key do |k|
        if k.respond_to?(:to_s)
          k = k.to_s
          nk = k[0,1] != '@' ? k : "@#{k}"
          self[nk]=self.delete(k)
        end
      end
    end
    self
  end

  # Aliases for better Extensions and Rails compatibility
  alias_method( :variablize_keys!, :keys_to_iv! )

end


#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#
# TODO
